from typing import Set, Optional, Union, List

from pydantic import BaseSettings


class Config(BaseSettings):
    # Your Config Here
    bwiki_navigator_command: Optional[str]
    bwiki_navigator_command_alias: Optional[Union[Set[str], List[str], str]]

    class Config:
        extra = "ignore"



from nonebot import get_driver
from nonebot.plugin import PluginMetadata

from .config import Config

global_config = get_driver().config
config = Config.parse_obj(global_config)

cmd_str = config.bwiki_navigator_command if config.bwiki_navigator_command else 'bwiki'
cmd_alias = config.bwiki_navigator_command_alias if config.bwiki_navigator_command_alias else set()
default_start = list(global_config.command_start)[0]

__plugin_meta__ = PluginMetadata(
    name='Bwiki Navigator',
    description='Bwiki助手（官方移植版）',
    usage=f'''欢迎使用Bwiki助手移植版，召唤方法如下：
{default_start}{cmd_str} bwiki子站域名 页面名(可选)
例如：{default_start}{cmd_str} clover 凯瑟琳
{default_start}{cmd_str} clover''',
    extra={'version': '0.1.0'}
)

from .handler import bwiki



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_bwiki_navigator']

package_data = \
{'': ['*']}

install_requires = \
['httpx[http2]>=0.20.0,<0.21.0',
 'nonebot-adapter-onebot>=2.0.0-beta.1,<3.0.0',
 'nonebot2>=2.0.0-beta.4,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-bwiki-navigator',
    'version': '0.1.0',
    'description': 'A nonebot2 plugin version of bwiki official bot',
    'long_description': '<div align="center">\n\n# nonebot-plugin-bwiki-navigator\n### BWiki助手Nonebot2插件移植版\n\n<a href="https://raw.githubusercontent.com/xzhouqd/nonebot-plugin-bwiki-navigator/main/LICENSE">\n    <img src="https://img.shields.io/github/license/xzhouqd/nonebot-plugin-help?style=for-the-badge" alt="license">\n</a>\n<a href="https://pypi.python.org/pypi/nonebot-plugin-bwiki-navigator">\n    <img src="https://img.shields.io/pypi/v/nonebot-plugin-bwiki-navigator?color=green&style=for-the-badge" alt="pypi">\n</a>\n<img src="https://img.shields.io/badge/python-^3.8-blue?style=for-the-badge" alt="python">\n<br />\n<img src="https://img.shields.io/badge/tested_python-3.10.4-blue?style=for-the-badge" alt="python">\n<img src="https://img.shields.io/static/v1?label=tested+env&message=go-cqhttp+1.0.0-rc3&color=blue&style=for-the-badge" alt="python">\n<br />\n<a href="https://github.com/botuniverse/onebot/blob/master/README.md">\n    <img src="https://img.shields.io/badge/Onebot-v11-brightgreen?style=for-the-badge" alt="onebot">\n</a>\n<a href="https://github.com/nonebot/nonebot2">\n    <img src="https://img.shields.io/static/v1?label=Nonebot&message=^2.0.0%2Dbeta.4&color=red&style=for-the-badge" alt="nonebot">\n</a>\n<a href="https://pypi.org/project/nonebot-adapter-cqhttp/">\n    <img src="https://img.shields.io/static/v1?label=Nonebot-adapters-onebot&message=^2.0.0%2Dbeta.1&color=red&style=for-the-badge" alt="nonebot-adapters-cqhttp">\n</a>\n</div>\n\n## 简介\n本插件是针对BWIKI助手解析的非官方Nonebot2插件移植版\n\n关于BWiki助手解析接入方式，请参阅 [BWIKI - WIKI助手文档](https://wiki.biligame.com/wiki/WIKI%E5%8A%A9%E6%89%8B%E6%96%87%E6%A1%A3)\n\n### 可配置项（可选）\n```\nBWIKI_NAVIGATOR_COMMAND="bwiki"                 // 本插件的主查询命令名\nBWIKI_NAVIGATOR_COMMAND_ALIAS=["wiki"]         // 本插件的查询命令别名列表\n```\n\n### 使用方法\n```\n/bwiki bwiki子站域名 页面名(可选)\n```\n\n### 示例\n```\n/bwiki clover\n\n四叶草剧场WIKI https://wiki.biligame.com/clover\n\n\n/bwiki clover 沙盒\n\n四叶草剧场WIKI 沙盒\nhttps://wiki.biligame.com/clover/?curid=19\n在这里测试你的代码！\n隐藏的wiki bot转换语句\n[图片]\n[图片]\n```\n\n### 已完整接入帮助菜单\n[nonebot-plugin-help](https://github.com/xzhouqd/nonebot-plugin-help) ([PyPI](https://pypi.python.org/pypi/nonebot-plugin-help))',
    'author': 'XZhouQD',
    'author_email': 'X.Zhou.QD@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/XZhouQD/nonebot-plugin-bwiki-navigator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import re
from typing import Dict

import httpx
from nonebot import on_command, get_driver
from nonebot.adapters import Event
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.matcher import Matcher
from nonebot.params import CommandArg, Arg

from . import cmd_str, cmd_alias, default_start
from .parser import SimpleInnerParser

bwiki = on_command(cmd_str, priority=1, aliases=cmd_alias)
bwiki.__help_name__ = cmd_str
bwiki.__help_info__ = f'''{default_start}{cmd_str} bwiki子站域名 页面名(可选)
例如：{default_start}{cmd_str} clover 凯瑟琳
{default_start}{cmd_str} clover'''


@bwiki.handle()
async def handle_first_receive(event: Event, matcher: Matcher, args: Message = CommandArg()):
    if args:
        matcher.set_arg("content", args)
    else:
        await matcher.finish(Message(MessageSegment.at(event.get_user_id()) + bwiki.__help_info__))


@bwiki.got("content")
async def get_result(event: Event, matcher: Matcher, content: Message = Arg()):
    at = MessageSegment.at(event.get_user_id())
    args = content.extract_plain_text().strip().split()[:2]
    if not args:
        await matcher.finish(Message(at + bwiki.__help_info__))
    site, sitename = await parse_sitename(args, at, matcher)
    if len(args) == 1:
        await matcher.finish(Message(at + f"{sitename} https://wiki.biligame.com/{site}"))

    page, page_part, result = await parse_page(args, at, matcher, site, sitename)
    if re.match("^(文件|file|File):.*\.(jpg|png|gif|jpeg)$", page):
        await parse_file_page(at, matcher, page, page_part, site)
    img_part, text_part = await parse_wikibot(result)
    await matcher.finish(Message(at + page_part + text_part + img_part))


async def parse_wikibot(result):
    to_feed = result["parse"]["text"]["*"]
    wikibot_parser = SimpleInnerParser("", ("class", "wiki-bot"))
    wikibot_parser.feed(to_feed)
    wikibot_parser.close()
    wikibot = '\n'.join(wikibot_parser.output)
    text_part = MessageSegment.text(wikibot)
    wikibot_img_parser = SimpleInnerParser("img", [("class", "wiki-bot-img"), ("src", None)], False)
    wikibot_img_parser.feed(to_feed)
    wikibot_img_parser.close()
    img_part = [MessageSegment.image(image_url) for image_url in wikibot_img_parser.output]
    return img_part, text_part


async def parse_file_page(at, matcher, page, page_part, site):
    async with httpx.AsyncClient(http2=True,
                                 base_url=f"https://wiki.biligame.com/{site}",
                                 follow_redirects=False) as client:
        response = await client.get("/api.php", params={"action": "parse", "format": "json",
                                                        "disablelimitreport": False, "text": f"[[{page}|link=]]"})
    img_parser = SimpleInnerParser("img", "src", False)
    img_parser.feed(response.json()["parse"]["text"]["*"])
    img_parser.close()
    img_part = MessageSegment.image(img_parser.output[0])
    await matcher.finish(Message(at + page_part + img_part))


async def parse_page(args, at, matcher, site, sitename):
    page = args[1]
    async with httpx.AsyncClient(http2=True,
                                 base_url=f"https://wiki.biligame.com/{site}",
                                 follow_redirects=False) as client:
        response = await client.get("/api.php", params={"action": "parse", "format": "json",
                                                        "disablelimitreport": False, "redirects": True,
                                                        "disableeditsection": True, "page": page})
    if response.status_code != 200:
        await matcher.finish(Message(at + f'解析站点 {site} 页面 {page} 失败'))
    result: Dict = response.json()
    if error := result.get("error", None):
        if error_code := error["code"] == "missingtitle":
            await matcher.finish(Message(at + f'解析站点 {site} 页面 {page} 失败：页面未找到'))
        else:
            await matcher.finish(Message(at + f'解析站点 {site} 页面 {page} 失败：{error_code}'))
    page = result["parse"]["title"]
    page_id = result["parse"]["pageid"]
    page_part = MessageSegment.text(f'{sitename} {page}\nhttps://wiki.biligame.com/{site}/?curid={page_id}\n')
    return page, page_part, result


async def parse_sitename(args, at, matcher):
    if site := args[0]:
        async with httpx.AsyncClient(http2=True,
                                     base_url=f"https://wiki.biligame.com/{site}",
                                     follow_redirects=False) as client:
            response = await client.get("/api.php", params={"action": "parse", "format": "json",
                                                            "disablelimitreport": False, "text": r"{{SITENAME}}"})
            if response.status_code != 200:
                await matcher.finish(Message(at + f'未找到站点 {site}，站点不存在或Bot解析失败'))
    sitename_parser = SimpleInnerParser("p")
    sitename_parser.feed(response.json()["parse"]["text"]["*"])
    sitename_parser.close()
    sitename = sitename_parser.output[0].strip().split('_')[0] if sitename_parser.output else site
    sitename = sitename if sitename else site
    return site, sitename



from html.parser import HTMLParser
from typing import Union, Tuple, List


class SimpleInnerParser(HTMLParser):
    def __init__(self, target_tag='p', target_attr: Union[str, Tuple, List[Tuple]] = None, text_context=True):
        super().__init__()
        self.output = []
        self.open_write = False
        self.target_tag = target_tag
        self.target_attr = target_attr
        self.text_context = text_context

    def handle_starttag(self, tag, attrs):
        if tag == self.target_tag or not self.target_tag:
            checker = get_from_attrs(attrs, self.target_attr) if self.target_attr else True
            self.open_write = True and checker
        if value := get_from_attrs(attrs, self.target_attr):
            if not self.text_context:
                self.output.append(str(value).strip())
                self.open_write = False

    def handle_endtag(self, tag):
        if tag == self.target_tag:
            self.open_write = False

    def handle_data(self, data: str):
        if self.open_write and self.text_context and data.strip():
            self.output.append(data.strip())


def get_from_attrs(attr_list, target):
    if not target:
        return False
    if isinstance(target, str):
        for attr in attr_list:
            if target == attr[0]:
                return attr[1]
    if isinstance(target, tuple):
        for attr in attr_list:
            if target[0] == attr[0] and target[1] in attr[1]:
                return True
    if isinstance(target, list) and len(target) == 2:
        find = target[0]
        fetch = target[1]
        got = False
        temp = None
        for attr in attr_list:
            if find[0] == attr[0] and find[1] in attr[1]:
                got = True
            if fetch[0] == attr[0]:
                temp = attr[1]
            if temp and got:
                return temp
    return False
